<?php

namespace Drupal\entity_pilot\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\EntityFormBuilderInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\UrlGeneratorInterface;
use Drupal\entity_pilot\AccountInterface;
use Drupal\entity_pilot\ArrivalInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides controller for Arrival page callbacks.
 */
class ArrivalController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * Entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  private $entityRepository;

  /**
   * URL Generator.
   *
   * @var \Drupal\Core\Routing\UrlGeneratorInterface
   */
  protected $urlGenerator;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('entity.form_builder'),
      $container->get('entity.repository'),
      $container->get('url_generator')
    );
  }

  /**
   * Constructs a new Arrival controller.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_manager
   *   Entity manager service.
   * @param \Drupal\Core\Entity\EntityFormBuilderInterface $entity_form
   *   Entity form service.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entityRepository
   *   Entity repository.
   * @param \Drupal\Core\Routing\UrlGeneratorInterface $urlGenerator
   *   Generator.
   */
  public function __construct(EntityTypeManagerInterface $entity_manager, EntityFormBuilderInterface $entity_form, EntityRepositoryInterface $entityRepository, UrlGeneratorInterface $urlGenerator) {
    $this->entityTypeManager = $entity_manager;
    $this->entityFormBuilder = $entity_form;
    $this->entityRepository = $entityRepository;
    $this->urlGenerator = $urlGenerator;
  }

  /**
   * Displays add arrival links for available accounts.
   *
   * @return array
   *   A render array for a list of the accounts that a arrival can be added
   *   for. If there is only one account, displays the add form for that
   *   account.
   */
  public function add() {
    $accounts = $this->entityTypeManager->getStorage('ep_account')->loadMultiple();
    // Only one account exists, display the add form for it.
    if ($accounts) {
      if (count($accounts) == 1) {
        $account = reset($accounts);
        return $this->addForm($account);
      }
      $account_links = [];
      /** @var \Drupal\entity_pilot\AccountInterface $account */
      foreach ($accounts as $account) {
        $account_links[$account->id()] = [
          'title' => $account->label(),
          'description' => $account->getDescription(),
          'url' => $account->toUrl('create-arrival'),
          'localized_options' => [],
        ];
      }
      return [
        '#theme' => 'admin_block_content',
        '#content' => $account_links,
      ];
    }
    else {
      return [
        [
          '#type' => 'markup',
          '#markup' => $this->t('No Entity Pilot accounts have been created.<br/>Please <a href=":url">create an account</a> first.', [
            ':url' => $this->urlGenerator->generateFromRoute('entity_pilot.account_add'),
          ]),
        ],
      ];
    }
  }

  /**
   * Presents the arrival creation form.
   *
   * @param \Drupal\entity_pilot\AccountInterface $ep_account
   *   Account to create the arrival against.
   *
   * @return array
   *   A form array as expected by drupal_render().
   */
  public function addForm(AccountInterface $ep_account) {
    $arrival = $this->entityTypeManager()->getStorage('ep_arrival')->create([
      'account' => $ep_account->id(),
    ]);
    return $this->entityFormBuilder()->getForm($arrival, 'add');
  }

  /**
   * Provides the page title for this controller.
   *
   * @param \Drupal\entity_pilot\AccountInterface $ep_account
   *   The account being used.
   *
   * @return string
   *   The page title.
   */
  public function getAddFormTitle(AccountInterface $ep_account) {
    return $this->t('Add arrival for %account', ['%account' => $ep_account->label()]);
  }

  /**
   * Provides the page title for this controller.
   *
   * @param \Drupal\entity_pilot\AccountInterface $ep_account
   *   The account being edited.
   *
   * @return string
   *   The page title.
   */
  public function titleAccountEdit(AccountInterface $ep_account) {
    return $this->t('Edit account %account', ['%account' => $ep_account->label()]);
  }

  /**
   * The _title_callback for the page that renders a single arrival.
   *
   * @param \Drupal\entity_pilot\ArrivalInterface $ep_arrival
   *   The current node.
   *
   * @return string
   *   The page title.
   */
  public function title(ArrivalInterface $ep_arrival) {
    return $this->entityRepository->getTranslationFromContext($ep_arrival)->label();
  }

  /**
   * The _title_callback for the page that renders arrival approve form.
   *
   * @param \Drupal\entity_pilot\ArrivalInterface $ep_arrival
   *   The current node.
   *
   * @return string
   *   The page title.
   */
  public function titleApprove(ArrivalInterface $ep_arrival) {
    return $this->t('Approve %label', [
      '%label' => $this->entityRepository->getTranslationFromContext($ep_arrival)->label(),
    ]);
  }

  /**
   * The _title_callback for the page that renders arrival edit form.
   *
   * @param \Drupal\entity_pilot\ArrivalInterface $ep_arrival
   *   The current node.
   *
   * @return string
   *   The page title.
   */
  public function titleEdit(ArrivalInterface $ep_arrival) {
    return $this->t('Edit %label', [
      '%label' => $this->entityRepository->getTranslationFromContext($ep_arrival)->label(),
    ]);
  }

  /**
   * The _title_callback for the page that renders arrival queue form.
   *
   * @param \Drupal\entity_pilot\ArrivalInterface $ep_arrival
   *   The current node.
   *
   * @return string
   *   The page title.
   */
  public function titleQueue(ArrivalInterface $ep_arrival) {
    return $this->t('Queue %label', [
      '%label' => $this->entityRepository->getTranslationFromContext($ep_arrival)->label(),
    ]);
  }

}
