<?php

namespace Drupal\entity_pilot\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Utility\Error;
use Drupal\entity_pilot\Exception\TransportException;
use Drupal\entity_pilot\SiteUrlTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the arrival edit forms.
 */
class ArrivalAddForm extends ArrivalForm {

  use SiteUrlTrait;

  /**
   * Entity Pilot transport service.
   *
   * @var \Drupal\entity_pilot\TransportInterface
   */
  protected $transport;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var self $instance */
    $instance = parent::create($container);
    $instance->transport = $container->get('entity_pilot.transport');
    $instance->urlGenerator = $container->get('url_generator');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $arrival = $this->entity;
    $form += parent::form($form, $form_state);
    // This is auto-created from the incoming flight.
    $form['info']['#access'] = FALSE;

    unset($form['status']);

    $form['search_container'] = [
      '#type' => 'container',
      '#weight' => -4,
      '#attributes' => [
        'class' => ['container-inline'],
      ],
    ];
    $form['search_container']['search'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search'),
      '#attributes' => [
        'placeholder' => $this->t('Title or Flight ID'),
      ],
      '#default_value' => $form_state->hasValue('search') ? $form_state->getValue('search') : NULL,
    ];
    $form['search_container']['go'] = [
      '#type' => 'submit',
      '#value' => $this->t('Search'),
      '#limit_validation_errors' => [[]],
      '#submit' => ['::search'],
    ];

    try {
      $flights = $this->transport->queryFlights($arrival->getAccount(), $form_state->hasValue('search') ? $form_state->getValue('search') : '');
      $storage = $form_state->get('storage');
      $storage['flights'] = $flights;
      $form_state->set('storage', $storage);
    }
    catch (TransportException $e) {
      $this->messenger()->addMessage($this->t('An error occurred connecting to the Entity Pilot backend, please try again later or visit the <a href="https://entitypilot.com/status">Entity Pilot status page</a>.'), 'error');
      $this->messenger()->addMessage($this->t('The message was: @message (@code)', [
        '@message' => $e->getMessage(),
        '@code' => $e->getCode(),
      ]), 'error');
      $variables = Error::decodeException($e);
      $this->logger->error($e->getMessage(), $variables);
      $form_state->set('entity_pilot_exception', $e);
      return [];
    }

    $options = [];
    $options['-1'] = [
      // Add an empty row to allow searching without selection.
      'id' => '-1',
      'info' => '',
      'contents' => '',
      'created' => '',
      '#attributes' => ['class' => ['visually-hidden']],
    ];
    foreach ($flights as $flight) {
      $date = new \DateTime($flight->getChanged());
      $options[$flight->getRemoteId()] = [
        'id' => $flight->getRemoteId(),
        'info' => $flight->getInfo(),
        'contents' => $this->formatPlural($flight->getCount(), '1 item', '@count items'),
        'created' => $date->format('d/m/Y'),
      ];
    }
    $header = [
      'id' => $this->t('ID'),
      'info' => $this->t('Info'),
      'contents' => $this->t('Contents'),
      'created' => $this->t('Created'),
    ];
    $form['remote_id'] = [
      '#type' => 'tableselect',
      '#weight' => -3,
      '#multiple' => FALSE,
      '#header' => $header,
      '#required' => TRUE,
      '#options' => $options,
      '#empty' => $this->t('No matching items'),
      '#caption' => $this->t('Select remote flight'),
      '#default_value' => -1,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    if ($form_state->get('entity_pilot_exception')) {
      return [];
    }
    return parent::actions($form, $form_state);
  }

  /**
   * Form submission handler for search button.
   */
  public function search($form, FormStateInterface &$form_state) {
    $form_state->setRebuild(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $arrival = $this->entity;
    $values = $form_state->getValues();
    $storage = $form_state->get('storage');
    if (!isset($values['remote_id']) ||
      !isset($storage['flights'][$values['remote_id']])) {
      $this->messenger()->addMessage($this->t('Please select a valid flight'));
      $form_state->setRebuild();
    }
    else {
      /* @var \Drupal\entity_pilot\Data\FlightManifestInterface $flight */
      $flight = $storage['flights'][$values['remote_id']];
      $form_state->setTemporaryValue('flight', $flight);
      unset($storage['flights']);
      $arrival->setInfo($flight->getInfo())
        ->setRemoteId($flight->getRemoteId())
        ->setRevisionLog($this->t('Fetched from Entity Pilot.'));
      parent::save($form, $form_state);
      $batch = [
        'operations' => [],
        'finished' => 'Drupal\entity_pilot\Batch\AirTrafficController::landed',
        'progress_message' => 'Fetching contents',
        'title' => $this->t('Fetching flight contents...'),
        'init_message' => $this->t('Please wait while we fetch and decrypt your content from Entity Pilot for you to review and approve...'),
      ];
      $batch['operations'][] = [
        'Drupal\entity_pilot\Batch\AirTrafficController::land',
        [$arrival, $flight, $this->getSite()],
      ];
      batch_set($batch);
      $form_state->setRedirect('entity.ep_arrival.approve_form', ['ep_arrival' => $arrival->id()]);
      $form_state->set('storage', $storage);
    }
  }

}
