<?php

namespace Drupal\entity_pilot\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\entity_pilot\FlightInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines a form for moving a arrival from pending to ready.
 */
class ArrivalApproveForm extends ArrivalForm {

  /**
   * The air traffic control service.
   *
   * @var \Drupal\entity_pilot\AirTrafficControlInterface
   */
  protected $airTrafficControl;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var self $instance */
    $instance = parent::create($container);
    $instance->airTrafficControl = $container->get('entity_pilot.air_traffic_control');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    unset($actions['delete']);
    $actions['submit']['#value'] = $this->t('Approve');
    $actions['queue'] = [
      '#type' => 'submit',
      '#value' => $this->t('Approve & Queue'),
      '#submit' => [
        '::submitForm',
        '::queue',
        '::save',
      ],
    ];
    $actions['land'] = [
      '#type' => 'submit',
      '#value' => $this->t('Approve & Land'),
      '#submit' => [
        '::submitForm',
        '::land',
      ],
    ];
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    /** @var \Drupal\entity_pilot\ArrivalInterface $arrival */
    $arrival = $this->entity;
    // Mark the transition as a new revision.
    $arrival->setNewRevision()
      ->setRevisionLog($this->t('Approved by @name', [
        '@name' => $this->currentUser()->getAccountName(),
      ]))
      ->setStatus(FlightInterface::STATUS_READY);
    return $arrival;
  }

  /**
   * {@inheritdoc}
   */
  public function queue(array $form, FormStateInterface $form_state) {
    $arrival = $this->entity;
    // Mark the transition as a new revision.
    $arrival->setNewRevision()
      ->setRevisionLog($this->t('Approved & queued by @name', [
        '@name' => $this->currentUser()->getAccountName(),
      ]))
      ->setStatus(FlightInterface::STATUS_QUEUED);
    return $arrival;
  }

  /**
   * {@inheritdoc}
   */
  public function land(array $form, FormStateInterface $form_state) {
    $arrival = $this->entity;
    // Mark the transition as a new revision.
    $arrival->setNewRevision()
      ->setRevisionLog($this->t('Approved and landed by @name', [
        '@name' => $this->currentUser()->getAccountName(),
      ]))
      ->setStatus(FlightInterface::STATUS_QUEUED);
    $batch = [
      'operations' => [],
      'finished' => 'Drupal\entity_pilot\Batch\CustomsOfficer::landingFinished',
      'progress_message' => 'Landed @current of @total',
      'title' => $this->t('Landing approved items...'),
    ];
    foreach ($arrival->getApproved() as $passenger_id) {
      $batch['operations'][] = [
        'Drupal\entity_pilot\Batch\CustomsOfficer::landPassenger', [
          $arrival,
          $passenger_id,
        ],
      ];
    }
    if (!empty($batch['operations'])) {
      if ($form_state->getValue('link_departure') && !$arrival->hasLinkedDeparture()) {
        $batch['operations'][] = [
          'Drupal\entity_pilot\Batch\AirTrafficController::linkDeparture', [
            $arrival,
          ],
        ];
      }
      batch_set($batch);
      $form_state->setRedirect('entity_pilot.arrival_list');
    }
    else {
      $this->messenger()->addMessage('No items were selected for approval', 'error');
      $form_state->setRebuild();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);
    $form['link_departure'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Create a matching departure'),
      '#description' => $this->t('Create a departure containing the same passengers. This will allow you to easily sync content in multiple directions.'),
      '#default_value' => $this->entity->hasLinkedDeparture(),
    ];
    return $form;
  }

}
