<?php

namespace Drupal\entity_pilot\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\entity_pilot\FlightInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the arrival edit forms.
 */
class ArrivalForm extends ContentEntityForm {

  /**
   * The arrival entity.
   *
   * @var \Drupal\entity_pilot\ArrivalInterface
   */
  protected $entity;

  /**
   * The Entity Pilot account storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $accountStorage;

  /**
   * The arrival entity storage.
   *
   * @var \Drupal\entity_pilot\ArrivalStorageInterface
   */
  protected $arrivalStorage;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $entity_manager = $container->get('entity_type.manager');
    /** @var self $instance */
    $instance = parent::create($container);
    $instance->accountStorage = $entity_manager->getStorage('ep_account');
    $instance->arrivalStorage = $entity_manager->getStorage('ep_arrival');
    $instance->logger = $container->get('logger.factory')->get('entity_pilot');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function prepareEntity() {
    $arrival = $this->entity;
    if (!$arrival->isNew()) {
      $arrival->setRevisionLog(NULL);
    }
    // Always use the default revision setting.
    $arrival->setNewRevision(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $arrival = $this->entity;
    $account = $this->currentUser();

    $form['status'] = [
      '#title' => $this->t('Status'),
      '#type' => 'item',
      '#markup' => $this->arrivalStorage->getAllowedStates()[$arrival->getStatus()],
    ];

    if ($arrival->getRemoteId()) {
      $form['remote_id_display'] = [
        '#title' => $this->t('Remote ID'),
        '#markup' => $arrival->getRemoteId(),
        '#type' => 'item',
      ];
    }

    // Add a log field if the "Create new revision" option is checked, or if the
    // current user has the ability to check that option.
    $form['revision_information'] = [
      '#type' => 'details',
      '#title' => $this->t('Revision information'),
      '#open' => FALSE,
      '#attributes' => [
        'class' => ['ep-arrival-form-revision'],
      ],
      '#attached' => [
        'library' => ['entity_pilot/drupal.arrival_form'],
      ],
      '#weight' => 30,
      '#access' => $arrival->isNewRevision() || $account->hasPermission('administer entity_pilot arrivals'),
    ];

    $form['revision_information']['revision'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Create new revision'),
      '#default_value' => $arrival->isNewRevision(),
      '#access' => $account->hasPermission('administer entity_pilot arrivals'),
    ];

    // Check the revision log checkbox when the log textarea is filled in.
    // This must not happen if "Create new revision" is enabled by default,
    // since the state would auto-disable the checkbox otherwise.
    if (!$arrival->isNewRevision()) {
      $form['revision_information']['revision']['#states'] = [
        'checked' => [
          'textarea[name="log"]' => ['empty' => FALSE],
        ],
      ];
    }

    $form['revision_information']['log'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Revision log message'),
      '#rows' => 4,
      '#default_value' => $arrival->getRevisionLog(),
      '#description' => $this->t('Briefly describe the changes you have made.'),
    ];

    return parent::form($form, $form_state, $arrival);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Build the arrival object from the submitted values.
    parent::submitForm($form, $form_state);
    /** @var \Drupal\entity_pilot\ArrivalInterface $arrival */
    $arrival = $this->entity;

    // Save as a new revision if requested to do so.
    if (!$form_state->isValueEmpty('revision')) {
      $arrival->setNewRevision();
    }
    // Reset status to pending.
    if (!$arrival->isPending()) {
      $arrival->setStatus(FlightInterface::STATUS_PENDING);
      // Resetting status to pending constitutes a new revision.
      $arrival->setNewRevision();
      // If no revision log is set, add one.
      if (!$arrival->getRevisionLog()) {
        $arrival->setRevisionLog($this->t('Status reset to pending by @name', [
          '@name' => $this->currentUser()->getAccountName(),
        ]));
      }
    }

    return $arrival;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $arrival = $this->entity;
    $insert = $arrival->isNew();
    $arrival->save();
    $watchdog_args = ['@type' => $arrival->bundle(), '%info' => $arrival->label()];
    $account = $this->accountStorage->load($arrival->bundle());
    $t_args = ['@type' => $account->label(), '%info' => $arrival->label()];

    if ($insert) {
      $this->logger->notice('@type: added %info.', $watchdog_args);
      $this->messenger()->addMessage($this->t('Arrival for @type account named %info has been created.', $t_args));
    }
    else {
      $this->logger->notice('@type: updated %info.', $watchdog_args);
      $this->messenger()->addMessage($this->t('Arrival for @type account named %info has been updated.', $t_args));
    }

    if ($arrival->id()) {
      $form_state->setValue('id', $arrival->id());
      $form_state->set('id', $arrival->id());
      $form_state->setRedirect('entity_pilot.arrival_list');
    }
    else {
      // In the unlikely case something went wrong on save, the arrival will
      // be rebuilt and arrival form redisplayed.
      $this->messenger()->addMessage($this->t('The arrival could not be saved.'), 'error');
      $form_state->setRebuild(TRUE);
    }
  }

}
