<?php

namespace Drupal\entity_pilot\Form;

use Drupal\Core\Entity\ContentEntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\entity_pilot\FlightInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for the departure edit forms.
 */
class DepartureForm extends ContentEntityForm {

  /**
   * The departure entity.
   *
   * @var \Drupal\entity_pilot\DepartureInterface
   */
  protected $entity;

  /**
   * The Entity Pilot account storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $accountStorage;

  /**
   * The departure entity storage.
   *
   * @var \Drupal\entity_pilot\DepartureStorageInterface
   */
  protected $departureStorage;

  /**
   * Logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $entity_manager = $container->get('entity_type.manager');
    /** @var self $instance */
    $instance = parent::create($container);
    $instance->accountStorage = $entity_manager->getStorage('ep_account');
    $instance->departureStorage = $entity_manager->getStorage('ep_departure');
    $instance->logger = $container->get('logger.factory')->get('entity_pilot');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  protected function prepareEntity() {
    $departure = $this->entity;
    if (!$departure->isNew()) {
      $departure->setRevisionLog(NULL);
    }
    // Always use the default revision setting.
    $departure->setNewRevision(TRUE);
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $departure = $this->entity;
    $account = $this->currentUser();

    if ($this->operation == 'edit') {
      $form['#title'] = $this->t('Edit departure %label', ['%label' => $departure->label()]);
    }

    $form['status'] = [
      '#title' => $this->t('Status'),
      '#type' => 'item',
      '#markup' => $this->departureStorage->getAllowedStates()[$departure->getStatus()],
    ];

    if ($remote_id = $departure->getRemoteId()) {
      $form['remote_id_show'] = [
        '#title' => $this->t('Remote ID'),
        '#markup' => $remote_id,
        '#type' => 'item',
      ];
      $form['remote_id'] = [
        '#type' => 'value',
        '#value' => $remote_id,
      ];
    }

    // Add a log field if the "Create new revision" option is checked, or if the
    // current user has the ability to check that option.
    $form['revision_information'] = [
      '#type' => 'details',
      '#title' => $this->t('Revision information'),
      '#open' => FALSE,
      '#attributes' => [
        'class' => ['ep-departure-form-revision'],
      ],
      '#attached' => [
        'library' => ['entity_pilot/drupal.departure_form'],
      ],
      '#weight' => 30,
      '#access' => $departure->isNewRevision() || $account->hasPermission('administer entity_pilot departures'),
    ];

    $form['revision_information']['revision'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Create new revision'),
      '#default_value' => $departure->isNewRevision(),
      '#access' => $account->hasPermission('administer entity_pilot departures'),
    ];

    // Check the revision log checkbox when the log textarea is filled in.
    // This must not happen if "Create new revision" is enabled by default,
    // since the state would auto-disable the checkbox otherwise.
    if (!$departure->isNewRevision()) {
      $form['revision_information']['revision']['#states'] = [
        'checked' => [
          'textarea[name="log"]' => ['empty' => FALSE],
        ],
      ];
    }

    $form['revision_information']['log'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Revision log message'),
      '#rows' => 4,
      '#default_value' => $departure->getRevisionLog(),
      '#description' => $this->t('Briefly describe the changes you have made.'),
    ];

    return parent::form($form, $form_state, $departure);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Build the departure object from the submitted values.
    parent::submitForm($form, $form_state);

    /** @var \Drupal\entity_pilot\DepartureInterface $departure */
    $departure = $this->entity;

    // Save as a new revision if requested to do so.
    if (!$form_state->isValueEmpty('revision')) {
      $departure->setNewRevision();
    }
    // Reset status to pending.
    if (!$departure->isPending()) {
      $departure->setStatus(FlightInterface::STATUS_PENDING);
      // Resetting status to pending constitutes a new revision.
      $departure->setNewRevision();
      // If no revision log is set, add one.
      if (!$departure->getRevisionLog()) {
        $departure->setRevisionLog($this->t('Status reset to pending by @name', [
          '@name' => $this->currentUser()->getAccountName(),
        ]));
      }
    }

    return $departure;
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $departure = $this->entity;
    $insert = $departure->isNew();
    $departure->save();
    $watchdog_args = ['@type' => $departure->bundle(), '%info' => $departure->label()];
    $account = $this->accountStorage->load($departure->bundle());
    $t_args = ['@type' => $account->label(), '%info' => $departure->label()];

    if ($insert) {
      $this->logger->notice('@type: added %info.', $watchdog_args);
      $this->messenger()->addMessage($this->t('Departure for @type account named %info has been created.', $t_args));
    }
    else {
      $this->logger->notice('@type: updated %info.', $watchdog_args);
      $this->messenger()->addMessage($this->t('Departure for @type account named %info has been updated.', $t_args));
    }

    if ($departure->id()) {
      $form_state->setValue('id', $departure->id());
      $form_state->set('id', $departure->id());
      $form_state->setRedirect('entity_pilot.departure_list');
      if ($insert) {
        $form_state->setRedirect('entity.ep_departure.approve_form', ['ep_departure' => $departure->id()]);
      }
    }
    else {
      // In the unlikely case something went wrong on save, the arrival will
      // be rebuilt and arrival form redisplayed.
      $this->messenger()->addMessage($this->t('The departure could not be saved.'), 'error');
      $form_state->setRebuild(TRUE);
    }
  }

}
