<?php

namespace Drupal\entity_pilot\Storage;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Cache\MemoryCache\MemoryCacheInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\Sql\SqlContentEntityStorage;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Queue\QueueInterface;
use Drupal\entity_pilot\BaggageHandlerInterface;
use Drupal\entity_pilot\DepartureInterface;
use Drupal\entity_pilot\DepartureStorageInterface;
use Drupal\entity_pilot\FlightInterface;
use Drupal\entity_pilot\Utility\FlightStub;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Serializer\Serializer;

/**
 * Provides storage for the 'ep_departure' entity type.
 */
class DepartureStorage extends SqlContentEntityStorage implements DepartureStorageInterface {

  /**
   * The serializer service.
   *
   * @var \Symfony\Component\Serializer\Serializer
   */
  protected $serializer;

  /**
   * The queue service.
   *
   * @var \Drupal\Core\Queue\QueueInterface
   */
  protected $queue;

  /**
   * The baggage handler service.
   *
   * @var \Drupal\entity_pilot\BaggageHandlerInterface
   */
  protected $baggageHandler;

  /**
   * Constructs a DepartureStorage object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection to be used.
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entity_field_manager
   *   The entity field manager.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache
   *   The cache backend to be used.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   * @param \Drupal\Core\Cache\MemoryCache\MemoryCacheInterface $memory_cache
   *   The memory cache backend to be used.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Symfony\Component\Serializer\Serializer $serializer
   *   The serializer service.
   * @param \Drupal\Core\Queue\QueueInterface $queue
   *   The entity pilot departures queue.
   * @param \Drupal\entity_pilot\BaggageHandlerInterface $baggage_handler
   *   The entity pilot baggage handler service.
   */
  public function __construct(EntityTypeInterface $entity_type, Connection $database, EntityFieldManagerInterface $entity_field_manager, CacheBackendInterface $cache, LanguageManagerInterface $language_manager, MemoryCacheInterface $memory_cache, EntityTypeBundleInfoInterface $entity_type_bundle_info, EntityTypeManagerInterface $entity_type_manager, Serializer $serializer, QueueInterface $queue, BaggageHandlerInterface $baggage_handler) {
    parent::__construct($entity_type, $database, $entity_field_manager, $cache, $language_manager, $memory_cache, $entity_type_bundle_info, $entity_type_manager);
    $this->serializer = $serializer;
    $this->queue = $queue;
    $this->baggageHandler = $baggage_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('database'),
      $container->get('entity_field.manager'),
      $container->get('cache.entity'),
      $container->get('language_manager'),
      $container->get('entity.memory_cache'),
      $container->get('entity_type.bundle.info'),
      $container->get('entity_type.manager'),
      $container->get('serializer'),
      $container->get('queue')->get('entity_pilot_departures'),
      $container->get('entity_pilot.baggage_handler')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getSchema() {
    $schema = parent::getSchema();

    // Add some indexes for status and remote id.
    $schema['entity_pilot_departure']['indexes']['status'] = ['status'];
    $schema['entity_pilot_departure']['indexes']['remote_id'] = ['remote_id'];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public function getAllowedStates() {
    return [
      FlightInterface::STATUS_PENDING => $this->t('Pending'),
      FlightInterface::STATUS_READY => $this->t('Ready'),
      FlightInterface::STATUS_QUEUED => $this->t('Queued'),
      FlightInterface::STATUS_LANDED => $this->t('Landed'),
      FlightInterface::STATUS_ARCHIVED => $this->t('Archived'),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getDependencies(DepartureInterface $entity) {
    $dependencies = [];
    $passenger_uuids = [];
    foreach ($entity->getPassengers() as $passenger) {
      $passenger_uuids[$passenger->uuid()] = $passenger->uuid();
      $dependencies += array_merge($dependencies, $this->baggageHandler->calculateDependencies($passenger));
    }
    return array_diff_key($dependencies, $passenger_uuids);
  }

  /**
   * {@inheritdoc}
   */
  public function queue(DepartureInterface $entity) {
    $this->queue->createItem(FlightStub::create($entity->getRevisionId()));
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function __sleep() {
    // @todo Remove this when https://www.drupal.org/node/2481729 is resolved.
    $this->_serviceIds = [];
    $vars = get_object_vars($this);
    foreach ($vars as $key => $value) {
      if (is_object($value) && isset($value->_serviceId)) {
        // If a class member was instantiated by the dependency injection
        // container, only store its ID so it can be used to get a fresh object
        // on unserialization.
        $this->_serviceIds[$key] = $value->_serviceId;
        unset($vars[$key]);
      }
      // Special case the container, which might not have a service ID.
      elseif ($value instanceof ContainerInterface) {
        $this->_serviceIds[$key] = 'service_container';
        unset($vars[$key]);
      }
    }

    unset($vars['queue']);
    return array_keys($vars);
  }

  /**
   * {@inheritdoc}
   */
  public function __wakeup() {
    // @todo Remove this when https://www.drupal.org/node/2481729 is resolved.
    $container = \Drupal::getContainer();
    foreach ($this->_serviceIds as $key => $service_id) {
      $this->$key = $container->get($service_id);
    }
    $this->_serviceIds = [];
    $this->queue = $container->get('queue')->get('entity_pilot_departures');
  }

}
